
# GraphQL Core Schema

This module automatically exposed a configurable GraphQL schema of your data within Drupal.    
The [Drupal GraphQL Module](https://www.drupal.org/project/graphql) in V3 has some issues:

- It's unmaintained and lacks PHP 8.1 support
- It exposes the whole schema of Drupal, but there is no easy overview / configuration what should be exposed and what not

We decided to build an automatic schema generator for [Drupal GraphQL Module](https://www.drupal.org/project/graphql) V4, that allows you to enable the part of the schema you need without writing custom code.
## What is implemented

- Entities (Nodes, User, Paragraphs, Commerce Entities, Custom Entities)
- Fields (all common fields are implemented, including Text, Boolean, List, Entity References, Address, Name)
- Media
- Breadcrumb
- Menu
- Routing (method `route()`)
- Formatted Dates
- Views (enable and query any view with Arguments)
- Entity Query (do any entity query with conditions, sorting and so on)
- Entity Display (fetch rendered fields based on a view mode)
- Field Configuration (query and field configuration)
- Language Switcher Links
- Local Tasks Links

The generated schema is similar to the schema generated by the V3 module but we did some major improvements:

- We removed the unnecessary intermedia step over `entity` when accessing entity references
- We stick to sane defaults and tried to avoid nullable parents as much as possible
    - Example: The body field does no longer have `body.processed` and `body.value`. You have a direct field called `body`and you optionally can pass a viewmode if you need the summary. But you always get a `string` even if the body field is not set / null in Drupal.
- The number of possible empty / null parents in the frontend should be drastically lower. Accessing an image on a media looks like this:

``` 
... on NodeTest {
  fieldMediaImageRef {
    fieldImage { uri } } 
}
```
## Included submodules

- graphql_metatag_schema ([Metatag support](https://www.drupal.org/project/metatag))
- graphql_tablefield_schema ([Tablefield support](https://www.drupal.org/project/tablefield))
- graphql_rokka_schema ([Rokka.io support](https://www.drupal.org/project/rokka))
- (in progress) Form API and Webform support

## Required patches for the GraphQL module
To support Views and Entity Queries, the GraphQL4 module needs the missing SubrequestBuffer class. A [merge request is open on GitHub](https://github.com/drupal-graphql/graphql/pull/1313).

You can add the patch to your composer.json like this:
```
 "drupal/graphql":  {       
   "Add missing subrequest buffer.": "https://patch-diff.githubusercontent.com/raw/drupal-graphql/graphql/pull/1313.patch"  
 },
```
Generating a large schema with a lot of schema extensions leads to a serious performance with the Drupal GraphQL v4 module. These issues have been outlined in this [GitHub issue](https://github.com/drupal-graphql/graphql/issues/1312). Until the related merge request is accepted, these improvements are part of this module and no additional patch is needed. As soon as they are merged, these changes will be removed from our module.

## Installation and configuration

- Enable the module and the needed submodules.
- Create a new GraphQL Server under  
  `/admin/config/graphql`
- In the Schema dropdown, select "Core composable Schema"
- Enable the needed Schema extensions
- Enable the needed Views
- Configure the exposed entity types

## Hint
- Disable the development mode of the GraphQL 4 module if you are not actively developing the schema. You will get a huge performance gain because the schema and the schema extensions will be cached.
- To extend and change the schema, you can write your one schema extension plugin and inherit from `schema = "core_composable"`
- If you find a bug or have a valuable contribution, please open an issue and create a patch / open a merge request. Contributions are very welcome.

## Roadmap
This module is currently in a **beta** state. Feel free to test it and give feedback, but expect some breaking changes in the upcoming weeks.
We are currently fine-tuning and improving the module in very short cycles.
A stable release is expected by the end of in Q1 / 2023.
