<?php

namespace Drupal\graphql_core_schema\Plugin\GraphQL\SchemaExtension;

use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\graphql\GraphQL\ResolverBuilder;
use Drupal\graphql\GraphQL\ResolverRegistryInterface;
use Drupal\graphql\Plugin\GraphQL\SchemaExtension\SdlSchemaExtensionPluginBase;
use Drupal\graphql_core_schema\CoreSchemaExtensionInterface;

/**
 * A schema extension to get an entity display.
 *
 * @SchemaExtension(
 *   id = "entity_display",
 *   name = "Entity Display",
 *   description = "An extension that provides entity form and view display fields.",
 *   schema = "core_composable"
 * )
 */
class EntityDisplayExtension extends SdlSchemaExtensionPluginBase implements CoreSchemaExtensionInterface {

  /**
   * {@inheritdoc}
   */
  public function getEntityTypeDependencies() {
    return ['entity_form_display', 'entity_view_display'];
  }

  /**
   * {@inheritdoc}
   */
  public function getExtensionDependencies() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function registerResolvers(ResolverRegistryInterface $registry): void {
    $builder = new ResolverBuilder();

    $registry->addFieldResolver('FieldConfig', 'fieldStorageDefinition',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->getFieldStorageDefinition();
      })
    );

    $registry->addFieldResolver('FieldDefinition', 'description',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->getDescription();
      })
    );

    $registry->addFieldResolver('FieldDefinition', 'name',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->getName();
      })
    );

    $registry->addFieldResolver('FieldDefinition', 'type',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->getType();
      })
    );

    $registry->addFieldResolver('FieldDefinition', 'targetEntityTypeId',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->getTargetEntityTypeId();
      })
    );
    $registry->addFieldResolver('FieldDefinition', 'targetBundle',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->getTargetBundle();
      })
    );
    $registry->addFieldResolver('FieldDefinition', 'isRequired',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->isRequired();
      })
    );
    $registry->addFieldResolver('FieldDefinition', 'isReadOnly',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->isReadOnly();
      })
    );
    $registry->addFieldResolver('FieldDefinition', 'isTranslatable',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->isTranslatable();
      })
    );
    $registry->addFieldResolver('FieldDefinition', 'uniqueIdentifier',
      $builder->callback(function (FieldDefinitionInterface $field) {
        return $field->getUniqueIdentifier();
      })
    );

    $registry->addTypeResolver('FieldDefinition', function ($value) {
      if ($value instanceof BaseFieldDefinition) {
        return 'BaseFieldDefinition';
      }
      return 'FieldConfig';
    });
  }

}
