<?php

namespace Drupal\graphql_core_schema\Plugin\GraphQL\SchemaExtension;

use Drupal\graphql\GraphQL\ResolverBuilder;
use Drupal\graphql\GraphQL\ResolverRegistryInterface;
use Drupal\graphql\Plugin\GraphQL\SchemaExtension\SdlSchemaExtensionPluginBase;

/**
 * A schema extensions for date formatting.
 *
 * @SchemaExtension(
 *   id = "formatted_date",
 *   name = "Formatted Date and Time",
 *   description = "Provides fields to get PHP and Drupal formatted dates.",
 *   schema = "core_composable"
 * )
 */
class FormattedDateExtension extends SdlSchemaExtensionPluginBase {

  /**
   * {@inheritdoc}
   */
  public function registerResolvers(ResolverRegistryInterface $registry): void {
    $builder = new ResolverBuilder();
    $composite = $builder->compose(
      $builder->produce('formatted_date')
        ->map('timestamp', $builder->fromParent())
        ->map('format', $builder->fromArgument('format'))
        ->map('drupalDateFormat', $builder->fromArgument('drupalDateFormat'))
    );

    $registry->addFieldResolver('FieldItemTypeChanged', 'formatted', $composite);
    $registry->addFieldResolver('FieldItemTypeCreated', 'formatted', $composite);
    $registry->addFieldResolver('FieldItemTypeTimestamp', 'formatted', $composite);
    $registry->addFieldResolver('FieldItemTypeDatetime', 'formatted', $composite);
  }

}
