<?php

namespace Drupal\graphql_core_schema\Plugin\GraphQL\SchemaExtension;

use Drupal\Core\Menu\MenuLinkInterface;
use Drupal\Core\Menu\MenuLinkTreeElement;
use Drupal\graphql\GraphQL\ResolverBuilder;
use Drupal\graphql\GraphQL\ResolverRegistryInterface;
use Drupal\graphql\Plugin\GraphQL\SchemaExtension\SdlSchemaExtensionPluginBase;
use Drupal\graphql_core_schema\CoreSchemaExtensionInterface;
use Drupal\menu_link_content\Plugin\Menu\MenuLinkContent;

/**
 * A schema extension to load menu links.
 *
 * @SchemaExtension(
 *   id = "menu",
 *   name = "Menu",
 *   description = "Load menu links.",
 *   schema = "core_composable"
 * )
 */
class MenuExtension extends SdlSchemaExtensionPluginBase implements CoreSchemaExtensionInterface {

  /**
   * {@inheritdoc}
   */
  public function getEntityTypeDependencies() {
    return ['menu', 'menu_link_content'];
  }

  /**
   * {@inheritdoc}
   */
  public function getExtensionDependencies() {
    return ['routing'];
  }

  /**
   * {@inheritdoc}
   */
  public function registerResolvers(ResolverRegistryInterface $registry): void {
    $builder = new ResolverBuilder();
    $registry->addTypeResolver('MenuLinkTreeElement', function ($value) {
      if ($value instanceof MenuLinkTreeElement) {
        return 'MenuLinkTreeElement';
      }
    });
    $registry->addTypeResolver('MenuLink', function ($value) {
      if ($value instanceof MenuLinkInterface) {
        return 'MenuLink';
      }
    });

    $registry->addFieldResolver('Menu', 'links', $builder->compose(
      $builder->produce('menu_links')->map('menu', $builder->fromParent()),
    ));

    $registry->addFieldResolver('MenuLinkTreeElement', 'link', $builder->compose(
      $builder->produce('menu_tree_link')->map('element', $builder->fromParent())
    ));

    $registry->addFieldResolver('MenuLinkTreeElement', 'subtree', $builder->compose(
      $builder->produce('menu_tree_subtree')->map('element', $builder->fromParent())
    ));

    $registry->addFieldResolver('MenuLink', 'label', $builder->compose(
      $builder->produce('menu_link_label')->map('link', $builder->fromParent())
    ));

    $registry->addFieldResolver('MenuLink', 'description', $builder->compose(
      $builder->produce('menu_link_description')->map('link', $builder->fromParent())
    ));

    $registry->addFieldResolver('MenuLink', 'expanded', $builder->compose(
      $builder->produce('menu_link_expanded')->map('link', $builder->fromParent())
    ));

    $registry->addFieldResolver('MenuLink', 'attribute', $builder->compose(
      $builder->produce('menu_link_attribute')
        ->map('link', $builder->fromParent())
        ->map('attribute', $builder->fromArgument('name'))
    ));

    $registry->addFieldResolver('MenuLink', 'url', $builder->compose(
      $builder->produce('menu_link_url')->map('link', $builder->fromParent())
    ));

    // Extract the menu_link_content entity if available.
    $registry->addFieldResolver('MenuLink', 'content', $builder->compose(
      $builder->callback(function ($value) {
        if ($value instanceof MenuLinkContent) {
          return $value->getDerivativeId();
        }
      }),
      $builder->produce('entity_load_by_uuid')
        ->map('type', $builder->fromValue('menu_link_content'))
        ->map('uuid', $builder->fromParent())
    ));
  }

}
